#!/bin/python

import sys, os, struct
import hashlib, ecdsa, random
from random import SystemRandom

randrange = SystemRandom().randrange

def s2i(s):
	result = 0L
	for c in s:
		result = 256 * result + ord(c)
	return result

def get_vsh_curves():
	curves = []
	curve_fmt = '>20s20s20s20s20s20s'

	with open('vsh_curves', 'rb') as in_file:
		file_data = in_file.read()
		file_size = len(file_data)
		num_curves = int(file_size / struct.calcsize(curve_fmt))
		for i in xrange(num_curves):
			data = file_data[i * struct.calcsize(curve_fmt):(i + 1) * struct.calcsize(curve_fmt)]
			inv_data = ''.join([chr((~ord(x)) & 0xff) for x in data])
			p, a, b, n, gx, gy = struct.unpack(curve_fmt, inv_data)
			curves.append({
				'p': s2i(p),
				'a': s2i(a),
				'b': s2i(b),
				'n': s2i(n),
				'gx': s2i(gx),
				'gy': s2i(gy)
			})

	return curves

def get_loader_curves():
	curves = []
	curve_fmt = '>20s20s20s21s20s20s'

	with open('ldr_curves', 'rb') as in_file:
		file_data = in_file.read()
		file_size = len(file_data)
		num_curves = int(file_size / struct.calcsize(curve_fmt))
		for i in xrange(num_curves):
			data = file_data[i * struct.calcsize(curve_fmt):(i + 1) * struct.calcsize(curve_fmt)]
			inv_data = ''.join([chr((~ord(x)) & 0xff) for x in data])
			p, a, b, n, gx, gy = struct.unpack(curve_fmt, inv_data)
			curves.append({
				'p': s2i(p),
				'a': s2i(a),
				'b': s2i(b),
				'n': s2i(n),
				'gx': s2i(gx),
				'gy': s2i(gy)
			})

	return curves

def get_file_contents(file_path):
	with open(file_path, 'rb') as in_file:
		return in_file.read()

def calculate_sha1_hash(data):
	return hashlib.sha1(data).digest()

#curves = get_loader_curves()
curves = get_vsh_curves()

def get_curve_parameters(type):
	params = curves[type]
	return params['p'], params['a'], params['b'], params['n'], params['gx'], params['gy']

curve_type = 0x02
k = 0x00bf21224b041f29549db25e9aade19e720a1fe0f1
#k = 0x00E5C4D0A8249A6F27E5E0C9D534F4DA15223F42AD
#k = 0x001AD976FCDE86F5B8FF3E63EF3A7F94E861975BA3

#qx = 0xA13AFE8B63F897DA2D3DC3987B39389DC10BAD99L
#qy = 0xDFB703838C4A0BC4E8BB44659C726CFD0CE60D0EL
       

data = get_file_contents('data')

p, a, b, n, gx, gy = get_curve_parameters(curve_type) # parameters
c = ecdsa.ellipticcurve.CurveFp(p, a, b) # curve equation
g = ecdsa.ellipticcurve.Point(c, gx, gy, n) # generator point
#q = ecdsa.ellipticcurve.Point(c, qx, qy)
e = s2i(calculate_sha1_hash(data)) 

public_key = ecdsa.ecdsa.Public_key(g, g * k)
private_key = ecdsa.ecdsa.Private_key(public_key, k)

# sign data
m = randrange(1, n)
signature = private_key.sign(e, m)

print 'sha1=', calculate_sha1_hash(data).encode('hex')
print 'e =', hex(e);
print 'm =', hex(m);
print 'p =', hex(p);
print 'a =', hex(a);
print 'b =', hex(b);
print 'N =', hex(n);
print 'Gx =', hex(gx);
print 'Gy =', hex(gy);
print 'k =', hex(k);
print 'R: {0:042X}'.format(signature.r)
print 'S: {0:042X}'.format(signature.s)

# verify data
print 'Status: OK' if public_key.verifies(e, signature) else 'Status: FAIL'
print 'Qx: {0:040X}'.format(public_key.point.x())
print 'Qy: {0:040X}'.format(public_key.point.y())
